#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>
//#include "common.h"

// комплексные типы данных C - C99, ISO/IEC 9899:1999 :
//---------------------------------------------------------

char *cput( complex c ) {
   static char scomp[ 40 ];
   sprintf( scomp, "%+.1f %+.1fi", creal( c ), cimag( c ) );
   return scomp;
}

#define print2(x) \
        printf( "комплексное: %s \tразмер = %d \tмодуль = %.2Lf\n", \
                cput(x), sizeof(x), cabsl(x) );

void test01( void ) {    // представление комплексных 
   double complex z1 = 1. + 1. * I;
   complex z2 = 3 - 4 * I;
   float complex  z3 = 4 - 3 * I;
   long double _Complex z4 = -3 + 3 * I;
   printf( "различные представления: \n" );
   print2( z1 );
   print2( z2 );
   print2( z3 );
   print2( z4 );
}

//---------------------------------------------------------

static complex z0  = 1. + 1. * I,
               z1  = 2. + 3. * I,
               z3  = 2.,
               z5  = -sqrt( 3. ) * I,
               z6  = 4 * I,
               z8  = -4 + I,
               z9  = -3 - 3 * I,
               z10 = sqrt( 2. ) - I,
               z11 = 2. - 3. * I,
               z21 = 3. + 2. * I;

inline char* pput( complex p, char* buf ) {
   sprintf( buf, "(%+.1f,%+.1f)", creal( p ), cimag( p ) );
   return buf;
}

void test02( void ) {           // разные представление комплексных 
   double PI = 4 * atan( 1.0 ); // можно просто M_PI
   void print( complex p ) {
      char buf[ 40 ];
      printf( "%s => abs = %.3f | arg = %.3f = %.2f*π = %.0f°\n",
           pput( p, buf ), cabs( p ),
           carg( p ), carg( p ) / PI, carg( p ) / PI * 180 );
   }
   printf( "abs * ( sin( arg ) + i * cos( arg ) ) || abs * exp( -i * arg ) :\n" );
   print( z0 );
   print( z1 );
   print( z5 );
   print( z8 );
   print( z9 );
   print( z10 );
}

//---------------------------------------------------------

void test03( void ) {           // 2D расстояния
   void print( complex p1, complex p2 ) {
      char buf[ 40 ];
      printf( "%s ... ", pput( p1, buf ) );
      printf( "%s => %.2f\n",
              pput( p2, buf ), cabs( p1 - p2 ) );
   };
   printf( "расстояния = cabs( p1 - p2 ) :\n" );
   print( z1, z21 );
   print( z1, z11 );
   print( z5, z6 );
   print( z8, z10 );
   print( z8, z9 );
}

//---------------------------------------------------------

void test04( void ) {           // 2D площадь
   void print( complex p1, complex p2 ) {
      double sq = cabs( p1 ) * cabs( p2 ) *
                  fabs( sin( carg( p1 ) - carg( p2 ) ) );  
      printf( "{ %s } & ", cput( p1 ) );
      printf( "{ %s } => %.3f\n", cput( p2 ), sq );
   };
   printf( "площадь = cabs( p1 ) * cabs( p2 ) * fabs( sin( carg( p1 ) - carg( p2 ) ) ) ) : \n" );
   print( z1, z3 );
   print( z1, z21 );
   print( z1, z11 );
   print( z5, z6 );
   print( z8, z10 );
   print( z8, z9 );
}

//---------------------------------------------------------

void ( *tests[] )( void ) = {          // последовательность тестов
        test01,                        // представления данных
        test02,                        // 2D расстояние
        test03,                        // 2D экспоненциальное представление
        test04,                        // 2D площадь
};
#include "main.c"
