#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// расширения GCC, Clang ...
//---------------------------------------------------------
static unsigned long long rdtsc( void ) {    // ассемблерные вставки  
   unsigned long long int x = 0;
#ifdef __i386__
   asm volatile ( "rdtsc" : "=A" (x) );      // команда RDTSC
#else
   asm volatile ( "rdtsc" );
   asm volatile ( "" : "=a" (x) );           // команда RDTSC
#endif
   return x;
}

void tgasm( void ) { 
   int i;
   for( i = 0; i < 3; i++ )
      printf( "регистр RDTSC=%llX\n", rdtsc() );
}
//---------------------------------------------------------
void tproc( void ) {                         // частота процессора
   time_t t1, t2;
   unsigned long long cf = 0, cs = 0; 
   printf( "GCC: инлайновые ассемблерные вставки\n" );
   time( &t1 );
   while( t1 == time( &t2 ) ) cf  = rdtsc(); // начало очередной секунды
   while( t2 == time( &t1 ) ) cs  = rdtsc(); // завершение этой секунды
   printf( "тактовая частота процессора %.3f Ghz\n",
           (double)( cs - cf ) / 1.E9 ); 
}
//---------------------------------------------------------
void tinc( void ) {           // вложенные функции
   int array[] = { 1, -2, 3, -4, 5, -6, 7 },
       size = sizeof( array ) / sizeof( *array ), i;
   void pow2( void ) {        // вложенное описание функции pow2()
      int decr( int arg ) {   // ещё один уровень вложенности функции decr()
         return arg - 1;
      }
      for( i = 0; i < size; i++ )
         array[ i ] = decr( array[ i ] ) * decr( array[ i ] );
   }
   printf( "вложенные функции GCC\n" );
   printf( "до\t:" );
   for( i = 0; i < size; i++ )
      printf( "%2d%s", array[ i ], ( i == size - 1 ? "\n" : " , " ) );
   pow2();
   printf( "после\t:" );
   for( i = 0; i < size; i++ )
      printf( "%2d%s", array[ i ], ( i == size - 1 ? "\n" : " , " ) );
}
//---------------------------------------------------------

void ( *tests[] )( void ) = {          // последовательность тестов
#ifdef GCC
        tgasm,                         // ассемблерные вставки: RDTSC
        tproc,                         // частота процессора
        tinc,                          // вложенные описания функций
#endif
};
#include "../main.c"

